# $Id: /local/Handel/trunk/t/lib/Handel/TestComponents/Validation.pm 1569 2007-06-24T15:35:46.298350Z claco  $
package Handel::TestComponents::Validation;
use strict;
use warnings;
use base qw/Handel::Components::Validation/;

1;
