# $Id: /local/Handel/trunk/t/lib/Handel/Checkout/Plugin/NotBlessedPlugin.pm 1569 2007-06-24T15:35:46.298350Z claco  $
package Handel::Checkout::Plugin::NotBlessedPlugin;
use strict;
use warnings;

sub new {
    return shift;
};

1;
