DELETE FROM orders;
DELETE FROM order_items;
INSERT INTO orders (id,shopper,type,billtofirstname,billtolastname,billtoaddress1,billtoaddress2,billtoaddress3,billtocity,billtostate,billtozip,billtocountry,billtodayphone,billtonightphone,billtofax,billtoemail,comments,created,handling,number,shipmethod,shipping,shiptosameasbillto,shiptofirstname,shiptolastname,shiptoaddress1,shiptoaddress2,shiptoaddress3,shiptocity,shiptostate,shiptozip,shiptocountry,shiptodayphone,shiptonightphone,shiptofax,shiptoemail,subtotal,total,updated, tax, custom) VALUES ('11111111-1111-1111-1111-111111111111','11111111-1111-1111-1111-111111111111',0,'Christopher','Laco','BillToAddress1','BillToAddress2','BillToAddress3','BillToCity','BillToState','BillToZip','BillToCountry','1-111-111-1111','2-222-222-2222','3-333-333-3333','mendlefarg@gmail.com','Comments','2005-07-15 20:12:34',8.95,'O123456789','UPS Ground',23.95,0,'Christopher','Laco','ShipToAddress1','ShipToAddress2','ShipToAddress3','ShipToCity','ShipToState','ShipToZip','ShipToCountry','4-444-444-4444','5-555-555-5555','6-666-666-6666','chrislaco@hotmail.com',5.55,37.95,'2005-07-16 20:12:34', 6.66, 'custom');
INSERT INTO orders (id,shopper,type,billtofirstname,billtolastname,billtoaddress1,billtoaddress2,billtoaddress3,billtocity,billtostate,billtozip,billtocountry,billtodayphone,billtonightphone,billtofax,billtoemail,comments,created,handling,number,shipmethod,shipping,shiptosameasbillto,shiptofirstname,shiptolastname,shiptoaddress1,shiptoaddress2,shiptoaddress3,shiptocity,shiptostate,shiptozip,shiptocountry,shiptodayphone,shiptonightphone,shiptofax,shiptoemail,subtotal,total,updated, tax, custom) VALUES ('22222222-2222-2222-2222-222222222222','11111111-1111-1111-1111-111111111111',1,'Christopher','Laco','BillToAddress1','BillToAddress2','BillToAddress3','BillToCity','BillToState','BillToZip','BillToCountry','1-111-111-1111','2-222-222-2222','3-333-333-3333','mendlefarg@gmail.com','Comments','2005-07-15 20:12:34',8.95,'O123456789','UPS Ground',23.95,0,'Christopher','Laco','ShipToAddress1','ShipToAddress2','ShipToAddress3','ShipToCity','ShipToState','ShipToZip','ShipToCountry','4-444-444-4444','5-555-555-5555','6-666-666-6666','chrislaco@hotmail.com',5.55,37.95,'2005-07-16 20:12:34', 6.66, 'custom');
INSERT INTO orders (id,shopper,type,billtofirstname,billtolastname,billtoaddress1,billtoaddress2,billtoaddress3,billtocity,billtostate,billtozip,billtocountry,billtodayphone,billtonightphone,billtofax,billtoemail,comments,created,handling,number,shipmethod,shipping,shiptosameasbillto,shiptofirstname,shiptolastname,shiptoaddress1,shiptoaddress2,shiptoaddress3,shiptocity,shiptostate,shiptozip,shiptocountry,shiptodayphone,shiptonightphone,shiptofax,shiptoemail,subtotal,total,updated, tax, custom) VALUES ('33333333-3333-3333-3333-333333333333','33333333-3333-3333-3333-333333333333',1,'Christopher','Laco','BillToAddress1','BillToAddress2','BillToAddress3','BillToCity','BillToState','BillToZip','BillToCountry','1-111-111-1111','2-222-222-2222','3-333-333-3333','mendlefarg@gmail.com','Comments','2005-07-15 20:12:34',8.95,'O123456789','UPS Ground',23.95,0,'Christopher','Laco','ShipToAddress1','ShipToAddress2','ShipToAddress3','ShipToCity','ShipToState','ShipToZip','ShipToCountry','4-444-444-4444','5-555-555-5555','6-666-666-6666','chrislaco@hotmail.com',5.55,37.95,'2005-07-16 20:12:34', 6.66, 'custom');
INSERT INTO order_items (id,orderid,sku,quantity,price,description, custom) VALUES ('11111111-1111-1111-1111-111111111111', '11111111-1111-1111-1111-111111111111','SKU1111',1,1.11,'Line Item SKU 1', 'custom');
INSERT INTO order_items (id,orderid,sku,quantity,price,description, custom) VALUES ('22222222-2222-2222-2222-222222222222', '11111111-1111-1111-1111-111111111111','SKU2222',2,2.22,'Line Item SKU 2', 'custom');
INSERT INTO order_items (id,orderid,sku,quantity,price,description, custom) VALUES ('33333333-3333-3333-3333-333333333333', '22222222-2222-2222-2222-222222222222','SKU3333',3,3.33,'Line Item SKU 3', 'custom');
INSERT INTO order_items (id,orderid,sku,quantity,price,description, custom) VALUES ('44444444-4444-4444-4444-444444444444', '33333333-3333-3333-3333-333333333333','SKU4444',4,4.44,'Line Item SKU 4', 'custom');
INSERT INTO order_items (id,orderid,sku,quantity,price,description, custom) VALUES ('55555555-5555-5555-5555-555555555555', '33333333-3333-3333-3333-333333333333','SKU1111',5,5.55,'Line Item SKU 5', 'custom');
