# $Id: /local/Handel/trunk/t/lib/Handel/Checkout/TestMessage.pm 1569 2007-06-24T15:35:46.298350Z claco  $
package Handel::Checkout::TestMessage;
use strict;
use warnings;
use base 'Handel::Checkout::Message';

1;